### 5.5 二項ロジスティック回帰 ###
### slide 5

d2 <- read.csv('chap05/input/data-attendance-2.txt')
head(d2)

### slide 7
d2$ratio <- d2$Y / d2$M

# データの分布を確認してみる
library(GGally)
d2$Af <- as.factor(d2$A)
ggpairs(d2[,c("Af","Score","M","Y","ratio")])

### slide8-9
library(rstan)
# とりあえず重回帰してみる
data5_4n <- list(N=nrow(d2), A=d2$A, Score=d2$Score/200, Y=d2$ratio)
model5_4n <- stan_model(file='model5_4_normal.stan')
fit5_4n <- sampling(model5_4n,data=data5_4n,seed=1234)
print(fit5_4n, pars=c('b1','b2','b3','sigma','lp__'),digits=3)

### slide10-11
# 予測分布の中央値と95%区間を求める
ms5_4n <- rstan::extract(fit5_4n)

d_qua5_4n <- t(apply(ms5_4n$y_pred, 2, quantile, prob=c(0.025, 0.5, 0.975)))
colnames(d_qua5_4n) <- c('p025', 'p50', 'p975')
d_qua5_4n <- data.frame(d2, d_qua5_4n)

# 予測分布の中央値を点、95%区間をバーで表す
coltmp <- c("blue", "red")
pchtmp <- c(4, 19)

plot(d_qua5_4n$ratio, d_qua5_4n$p50,
     xlim=c(0.4,1), ylim=c(0.3,1.1), 
     col=coltmp[(d_qua5_4n$A+1)],
     pch=pchtmp[(d_qua5_4n$A+1)],
     xlab="observed",
     ylab="predicted",
     bty="l")
arrows(x0=d_qua5_4n$ratio, y0=d_qua5_4n$p50,
       x1=d_qua5_4n$ratio, y1=d_qua5_4n$p975,
       length=0,col=coltmp[(d_qua5_4n$A+1)])
arrows(x0=d_qua5_4n$ratio, y0=d_qua5_4n$p50,
       x1=d_qua5_4n$ratio, y1=d_qua5_4n$p025,
       length=0,col=coltmp[(d_qua5_4n$A+1)])
abline(a=0,b=1,lty=2)
abline(h=1, col="black",lty=2,lwd=2)
legend("topleft",
       legend=c("A=0","A=1"),
       col=coltmp,
       pch=pchtmp)

### slide 13
# ロジスティック変換やってみる
logistic <- function(x) 1/(1+exp(-x)) # ロジスティック関数作成
test <- seq(-10, 10, 0.1)
plot(test, logistic(test), bty="l")
abline(v=0,lty=2)

summary(logistic(test))

### slide 14
# ロジスティック曲線の描画
par(mfrow=c(1,2))
plot(test, logistic(test),
    bty="l",type="l",lwd=2,
    xlim=c(-10,10),ylim=c(0,1),
    main="b1 = 0, b2 = ?",
    ylab="y",
    xlab="x")
abline(v=0,lty=2)
lines(test, logistic(-1*test),col="red",lwd=2)
lines(test, logistic(5*test),col="blue",lwd=2)
legend("topleft",lty=1,lwd=2,
      col=c("red","black","blue"),
      legend=c("b2 = -1", "b2 = 1", "b2 = 5"))

plot(test, logistic(test),
    bty="l",type="l",lwd=2,
    xlim=c(-10,10),ylim=c(0,1),
    main="b1 = ?, b2 = 0",
    ylab="y",
    xlab="x")
abline(v=0,lty=2)
lines(test, logistic(-3+test),col="red",lwd=2)
lines(test, logistic(3+test),col="blue",lwd=2)
legend("topleft",lty=1,lwd=2,
       col=c("red","black","blue"),
       legend=c("b1 = -3", "b1 = 1", "b1 = 3"))
par(mfrow=c(1,1))

### slide 17-18
# 二項ロジスティック回帰
data5_4b <- list(N=nrow(d2),A=d2$A, Score=d2$Score/200, Y=d2$Y, M=d2$M)
model5_4b <- stan_model(file='model5_4_binomial.stan')
fit5_4b <- sampling(model5_4b, data=data5_4b, seed=1234)
print(fit5_4b, pars=c('b1','b2','b3','lp__'),digits=3)

### slide 19
# 予測分布の中央値と95%区間を求める
ms5_4b <- rstan::extract(fit5_4b)

d_quan5_4b <- t(apply(ms5_4b$y_pred, 2, quantile, prob=c(0.025, 0.5, 0.975)))
colnames(d_quan5_4b) <- c('p025', 'p50', 'p975')
d_quan5_4b <- data.frame(d2, d_quan5_4b)

# 予測分布の中央値を点、95%区間をバーで表す
plot(d_quan5_4b$Y, d_quan5_4b$p50,xlim=c(0,80), ylim=c(0,80), 
     col=coltmp[(d_qua5_4n$A+1)],
     pch=pchtmp[(d_qua5_4n$A+1)],
     xlab="observed",
     ylab="predicted",
     bty="l")
arrows(x0=d_quan5_4b$Y, y0=d_quan5_4b$p50,
       x1=d_quan5_4b$Y, y1=d_quan5_4b$p975,
       length=0,col=coltmp[(d_quan5_4b$A+1)])
arrows(x0=d_quan5_4b$Y, y0=d_quan5_4b$p50,
       x1=d_quan5_4b$Y, y1=d_quan5_4b$p025,
       length=0,col=coltmp[(d_quan5_4b$A+1)])
abline(a=0,b=1,lty=2)
legend("topleft",
       legend=c("A=0","A=1"),
       col=coltmp,
       pch=pchtmp)

### 5.3 ロジスティック回帰 ###
# slide 25
d3 <- read.csv('chap05/input/data-attendance-3.txt')
head(d3)

# slide 30
# 置き換え可視化
# 値と天気の文字列を対応させたベクトルを作成
conv <- c(0, 0.2, 1)
names(conv) <- c('A', 'B', 'C')
conv

# d3の天気をちょっと読み込んでみる
head(d3$Weather)
# ↑を使ってconvの値を呼び出してみる
conv[head(d3$Weather)]

# slide31-32
# ロジスティック回帰
data5_5 <- list(I=nrow(d3), A=d3$A, Score=d3$Score/200, W=conv[d3$Weather], Y=d3$Y)
model5_5 <- stan_model(file='model5_5.stan')
fit5_5 <- sampling(model5_5, data=data5_5, seed=1234)
print(fit5_5, pars=c('b[1]','b[2]','b[3]','b[4]','lp__'),digits=3)

# slide 33
# model check figure 5_9
ms5_5 <- rstan::extract(fit5_5)

logistic <- function(x) 1/(1+exp(-x))
X <- 30:200

par(mfrow = c(2,3))

# A=0 & Weather=A
q_qua0A <- logistic(t(sapply(1:length(X), function(i) {
  q_mcmc <- ms5_5$b[,1] + 0*ms5_5$b[,2] + ms5_5$b[,3]*X[i]/200 + 0*ms5_5$b[,4]
  quantile(q_mcmc, probs=c(0.1, 0.5, 0.9))
})))
d_est0A <- data.frame(X, q_qua0A)
colnames(d_est0A) <- c('X', 'p10', 'p50', 'p90')

d3tmp0A <- subset(d3, (d3$A==0 & d3$Weather=="A"))

plot(d_est0A$X, d_est0A$p50, type="l", xlim=c(30,200), ylim=c(-0.1,1.1),
     bty="l", xlab="Score", ylab="q", main="A = 0, Weather = A")
lines(d_est0A$X, d_est0A$p10, lty=2)
lines(d_est0A$X, d_est0A$p90, lty=2)
points(d3tmp0A$Score, (d3tmp0A$Y+rnorm(nrow(d3tmp0A),0,0.03)))


# A=0 & Weather=B
q_qua0B <- logistic(t(sapply(1:length(X), function(i) {
  q_mcmc <- ms5_5$b[,1] + 0*ms5_5$b[,2] + ms5_5$b[,3]*X[i]/200 + 0.2*ms5_5$b[,4]
  quantile(q_mcmc, probs=c(0.1, 0.5, 0.9))
})))
d_est0B <- data.frame(X, q_qua0B)
colnames(d_est0B) <- c('X', 'p10', 'p50', 'p90')

d3tmp0B <- subset(d3, (d3$A==0 & d3$Weather=="B"))

plot(d_est0B$X, d_est0B$p50, type="l", xlim=c(30,200), ylim=c(-0.1,1.1),
     bty="l", xlab="Score", ylab="q", main="A = 0, Weather = B")
points(d_est0B$X, d_est0B$p10, type="l", lty=2)
points(d_est0B$X, d_est0B$p90, type="l", lty=2)
points(d3tmp0B$Score, (d3tmp0B$Y+rnorm(nrow(d3tmp0B),0,0.03)))


# A=0 & Weather=C
q_qua0C <- logistic(t(sapply(1:length(X), function(i) {
  q_mcmc <- ms5_5$b[,1] + 0*ms5_5$b[,2] + ms5_5$b[,3]*X[i]/200 + 1*ms5_5$b[,4]
  quantile(q_mcmc, probs=c(0.1, 0.5, 0.9))
})))
d_est0C <- data.frame(X, q_qua0C)
colnames(d_est0C) <- c('X', 'p10', 'p50', 'p90')

d3tmp0C <- subset(d3, (d3$A==0 & d3$Weather=="C"))

plot(d_est0C$X, d_est0C$p50, type="l", xlim=c(30,200), ylim=c(-0.1,1.1),
     bty="l", xlab="Score", ylab="q", main="A = 0, Weather = C")
points(d_est0C$X, d_est0C$p10, type="l", lty=2)
points(d_est0C$X, d_est0C$p90, type="l", lty=2)
points(d3tmp0C$Score, (d3tmp0C$Y+rnorm(nrow(d3tmp0C),0,0.03)))


# A=1 & Weather=A
q_qua1A <- logistic(t(sapply(1:length(X), function(i) {
  q_mcmc <- ms5_5$b[,1] + 1*ms5_5$b[,2] + ms5_5$b[,3]*X[i]/200 + 0*ms5_5$b[,4]
  quantile(q_mcmc, probs=c(0.1, 0.5, 0.9))
})))
d_est1A <- data.frame(X, q_qua1A)
colnames(d_est1A) <- c('X', 'p10', 'p50', 'p90')

d3tmp1A <- subset(d3, (d3$A==1 & d3$Weather=="A"))

plot(d_est1A$X, d_est1A$p50, type="l", xlim=c(30,200), ylim=c(-0.1,1.1),
     bty="l", xlab="Score", ylab="q", main="A = 1, Weather = A")
points(d_est1A$X, d_est1A$p10, type="l", lty=2)
points(d_est1A$X, d_est1A$p90, type="l", lty=2)
points(d3tmp1A$Score, (d3tmp1A$Y+rnorm(nrow(d3tmp1A),0,0.03)))


# A=1 & Weather=B
q_qua1B <- logistic(t(sapply(1:length(X), function(i) {
  q_mcmc <- ms5_5$b[,1] + 1*ms5_5$b[,2] + ms5_5$b[,3]*X[i]/200 + 0.2*ms5_5$b[,4]
  quantile(q_mcmc, probs=c(0.1, 0.5, 0.9))
})))
d_est1B <- data.frame(X, q_qua1B)
colnames(d_est1B) <- c('X', 'p10', 'p50', 'p90')

d3tmp1B <- subset(d3, (d3$A==0 & d3$Weather=="B"))

plot(d_est1B$X, d_est1B$p50, type="l", xlim=c(30,200), ylim=c(-0.1,1.1),
     bty="l", xlab="Score", ylab="q", main="A = 1, Weather = B")
points(d_est1B$X, d_est1B$p10, type="l", lty=2)
points(d_est1B$X, d_est1B$p90, type="l", lty=2)
points(d3tmp1B$Score, (d3tmp1B$Y+rnorm(nrow(d3tmp1B),0,0.03)))


# A=1 & Weather=C
q_qua1C <- logistic(t(sapply(1:length(X), function(i) {
  q_mcmc <- ms5_5$b[,1] + 1*ms5_5$b[,2] + ms5_5$b[,3]*X[i]/200 + 1*ms5_5$b[,4]
  quantile(q_mcmc, probs=c(0.1, 0.5, 0.9))
})))
d_est1C <- data.frame(X, q_qua1C)
colnames(d_est1C) <- c('X', 'p10', 'p50', 'p90')

d3tmp1C <- subset(d3, (d3$A==1 & d3$Weather=="C"))

plot(d_est1C$X, d_est1C$p50, type="l", xlim=c(30,200), ylim=c(-0.1,1.1),
     bty="l", xlab="Score", ylab="q", main="A = 1, Weather = C")
points(d_est1C$X, d_est1C$p10, type="l", lty=2)
points(d_est1C$X, d_est1C$p90, type="l", lty=2)
points(d3tmp1C$Score, (d3tmp1C$Y+rnorm(nrow(d3tmp1C),0,0.03)))

par(mfrow=c(1,1))

# model check ここまで

# slide 35
d_quan5_5 <- t(apply(ms5_5$q, 2, quantile, prob=c(0.025, 0.5, 0.975)))
colnames(d_quan5_5) <- c('p025', 'p50', 'p975')
d_quan5_5 <- data.frame(d3, d_quan5_5)

library(vioplot)
vioplot(subset(d_quan5_5,d_quan5_5$Y==0)$p50,
        subset(d_quan5_5,d_quan5_5$Y==1)$p50,
        names=c("Y=0","Y=1"),
        col="grey",
        ylim=c(0.4,1.0),
        horizontal=T)

# slide 36
# 著者のコードをほぼそのまま実行したので省略

# slide 38-39
# 練習問題4
test <- d3$Weather == "A"
head(d3$Weather)
head(test)
head(as.numeric(test))

data5_5p <- list(I=nrow(d3), A=d3$A, Score=d3$Score/200,
                Cloud=as.numeric(d3$Weather=="B"), Rain=as.numeric(d3$Weather=="C"),
                Y=d3$Y)
model5_5p <- stan_model(file='model5_5_ex1.stan')
fit5_5p <- sampling(model5_5p, data=data5_5p, seed=1234)
print(fit5_5p, pars=c('b[1]','b[2]','b[3]','b[4]','b[5]','lp__'),digits=3)

# slide 41-42
tmp <- c(1,2,3)
names(tmp) <- c("A","B","C")

# 練習問題4(ヒント活用)
data5_5p2 <- list(I=nrow(d3), 
                  A=d3$A, Score=d3$Score/200,W=tmp[d3$Weather],Y=d3$Y)
model5_5p2 <- stan_model(file='model5_5_ex2.stan')
fit5_5p2 <- sampling(model5_5p2, data=data5_5p2, seed=1234)
print(fit5_5p2, pars=c('b[1]','b[2]','b[3]','bw2','bw3','lp__'),digits=3)

### 5.4 ポアソン回帰
# slide 46-47
# とりあえず重回帰
data5_6n <- list(N=nrow(d2), A=d2$A, Score=d2$Score/200, M=d2$M)
model5_6n <- stan_model(file='model5_6_normal.stan')
fit5_6n <- sampling(model5_6n, data=data5_6n, seed=1234)
print(fit5_6n, pars=c('b[1]','b[2]','b[3]', 'sigma','lp__'),digits=3)

ms5_6n <- rstan::extract(fit5_6n)

d_qua5_6n <- t(apply(ms5_6n$m_pred, 2, quantile, prob=c(0.05, 0.5, 0.95)))
colnames(d_qua5_6n) <- c('p05', 'p50', 'p95')
d_qua5_6n <- data.frame(d2, d_qua5_6n)

summary(d_qua5_6n$p50)

# slide 49-50
# ポアソン回帰
data5_6p <- list(N=nrow(d2), A=d2$A, Score=d2$Score/200, M=d2$M)
model5_6p <- stan_model(file='model5_6_poisson.stan')
fit5_6p <- sampling(model5_6p, data=data5_6p, seed=1234)
print(fit5_6p, pars=c('b[1]','b[2]','b[3]','lp__'),digits=3)

ms5_6p <- rstan::extract(fit5_6p)

d_qua5_6p <- t(apply(ms5_6p$m_pred, 2, quantile, prob=c(0.05, 0.5, 0.95)))
colnames(d_qua5_6p) <- c('p05', 'p50', 'p95')
d_qua5_6p <- data.frame(d2, d_qua5_6p)

summary(d_qua5_6p$p50)

# slide 52
# 練習問題5

plot(d_qua5_6p$M, d_qua5_6p$p50,
     xlim=c(min(d_qua5_6p$M),max(d_qua5_6p$M)), ylim=c(min(d_qua5_6p$p05),max(d_qua5_6p$p95)),
     col=coltmp[(d_qua5_6p$A+1)],
     pch=pchtmp[(d_qua5_6p$A+1)],
     xlab="observed",
     ylab="predicted",
     bty="l")
arrows(x0=d_qua5_6p$M, y0=d_qua5_6p$p50,
       x1=d_qua5_6p$M, y1=d_qua5_6p$p95, 
       length=0,col=coltmp[(d_qua5_6p$A+1)])
arrows(x0=d_qua5_6p$M, y0=d_qua5_6p$p50,
       x1=d_qua5_6p$M, y1=d_qua5_6p$p05, 
       length=0,col=coltmp[(d_qua5_6p$A+1)])
abline(a=0,b=1,lty=2)
legend("topleft",
       legend=c("A=0","A=1"),
       col=coltmp,
       pch=pchtmp)


### 練習問題(6)
# slide 57
dex6 <- read.csv('chap05/input/data3a.csv')
head(dex6)

# slide 59
# 久保(2012)に従ってglm関数を使った分析
glm6 <- glm(y ~ x + f, data=dex6, family=poisson)
summary(glm6)

# slide 60-61
# stanで推定
data_ex6 <- list(N=nrow(dex6), X=(dex6$x), F=as.numeric(dex6$f=="T"), Y=dex6$y)
model_ex6 <- stan_model(file='model_ex6.stan')
fit_ex6 <- sampling(model_ex6, data=data_ex6, seed=1234)
print(fit_ex6, pars=c('b[1]','b[2]','b[3]','lp__'),digits=3)


### 練習問題7
# slide 62
dex7 <- read.csv('chap05/input/data4a.csv')
head(dex7)

# slide 64
# 久保(2012)に従ってglm関数を使って分析
glm7 <- glm(cbind(y, N-y) ~ x + f, data=dex7, family=binomial)
summary(glm7)

# slide 65-66
# stanで推定
data_ex7 <- list(I=nrow(dex7), X=dex7$x, F=as.numeric(dex7$f=="T"), N=dex7$N, Y=dex7$y)
model_ex7 <- stan_model(file='model_ex7.stan')
fit_ex7 <- sampling(model_ex7, data=data_ex7, seed=1234)
print(fit_ex7, pars=c("b[1]","b[2]","b[3]",'lp__'), digits=3)

# slide 67
# 観測値と予測値をプロット
ms_ex7 <- rstan::extract(fit_ex7)

d_quaex7 <- t(apply(ms_ex7$y_pred, 2, quantile, prob=c(0.05, 0.5, 0.95)))
colnames(d_quaex7) <- c('p05', 'p50', 'p95')
d_quaex7 <- data.frame(dex7, d_quaex7)

plot(d_quaex7$y, d_quaex7$p50,
     xlim=c(min(d_quaex7$y),max(d_quaex7$y)), ylim=c(min(d_quaex7$p05),max(d_quaex7$p95)),
     col=coltmp[(as.numeric(d_quaex7$f=="T")+1)],
     pch=pchtmp[(as.numeric(d_quaex7$f=="T")+1)],
     xlab="observed",
     ylab="predicted",
     bty="l")
arrows(x0=d_quaex7$y, y0=d_quaex7$p50,
       x1=d_quaex7$y, y1=d_quaex7$p95, 
       length=0,col=coltmp[(as.numeric(d_quaex7$f=="T")+1)])
arrows(x0=d_quaex7$y, y0=d_quaex7$p50,
       x1=d_quaex7$y, y1=d_quaex7$p05, 
       length=0,col=coltmp[(as.numeric(d_quaex7$f=="T")+1)])
abline(a=0,b=1,lty=2)
legend("topleft",
       legend=c("F=C","F=T"),
       col=coltmp,
       pch=pchtmp)